/*  spiasm.c  -  library functions for spi using PASM

*/

#include "spiasm.h"

//Constants used in spi_out() and spi_in() 
#define SHIFTOUT 1  
#define SHIFTIN 2

// function prototype

void spi_setcommand(spia_t *device, int cmd, int *argptr);
  
// These are the actual functions 
//  ****************************************
// Start cog, load ClockDelay and ClockState into Pasm variables in cog

spia_t *spi_start(int clkd_ns, int clks)
{
  spia_t *device;                                   // Declare spiasm pointer
  device = (void *) malloc(sizeof(spia_t));         // Allocate memory for it  
  if(device->cog == 0)
  {
    device->command = 0;                          // keep pasm in waiting loop at start till non zero
    extern int binary_xSPIAsm_dat_start[];
    device->cog = 1 + cognew((void*)binary_xSPIAsm_dat_start, (void*)device);
  }
  int clkd = (clkd_ns - 250) / 50;        // Calculate ClockDelay from nanosec delay value
  device->command = (4 << 16) | (clkd << 2) | clks ; // set ClockDelay and ClockState pasm variables 
  while(device->command);                            // wait till pasm resets command to zero
  return device;                                     // Return pointer to structure
}
  
//  ************************************
//  Stop and release cog

void spi_stop(spia_t *device)
{
  if(device->cog > 0)                                  // If cog running
  {
    cogstop(device->cog - 1);                          // Shut down cog
    device->cog = 0;                                   // Clear cog varaible
    free(device);                                      // Release allocated memory
  }  
}
  
//  ************************************
//  ** sends data to SPI device

void spi_out(spia_t *device, int DQ, int CLK, int MD, int BIT, int VLU)
{
  device->Dpin = DQ;            // load variable values for pasm
  device->Cpin = CLK;
  device->Mode = MD;
  device->Bits =  BIT;
  device->Value = VLU;
  spi_setcommand(device, SHIFTOUT, &device->Dpin);     
} 
  
//  ************************************
//  ** gets data from SPi device

int spi_in(spia_t *device, int DQ, int CLK, int MD, int BIT)
{
  device->Dpin = DQ;
  device->Cpin = CLK;
  device->Mode = MD;
  device->Bits = BIT;
  device->Flag = 1;
  spi_setcommand(device, SHIFTIN, &device->Dpin);  
  while(device->Flag);                   // pasm sets Flag = 0 when all bits in Value are read
  return device->Value;
}
  
//  ************************************
//  send packed command to shiftin or shiftout with address of variable Dpin to pasm

void spi_setcommand(spia_t *device, int cmd, int *argptr)  
{
  device->command = (int) argptr | (cmd << 16);
  while(device->command);
}